@@ -1,26 +1,26 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2014, Magnus Edenhill
+ * Copyright (c) 2014-2022, Magnus Edenhill
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met: 
- * 
+ * modification, are permitted provided that the following conditions are met:
+ *
  * 1. Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimer. 
+ *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution. 
- * 
+ *    and/or other materials provided with the distribution.
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
@@ -29,7 +29,7 @@
 /**
  * Apache Kafka consumer & producer example programs
  * using the Kafka driver from librdkafka
- * (https://github.com/edenhill/librdkafka)
+ * (https://github.com/confluentinc/librdkafka)
  */
 
 #include <iostream>
@@ -54,28 +54,26 @@
 #include "rdkafkacpp.h"
 
 
-static void metadata_print (const std::string &topic,
-                            const RdKafka::Metadata *metadata) {
+static void metadata_print(const std::string &topic,
+                           const RdKafka::Metadata *metadata) {
   std::cout << "Metadata for " << (topic.empty() ? "" : "all topics")
-           << "(from broker "  << metadata->orig_broker_id()
-           << ":" << metadata->orig_broker_name() << std::endl;
+            << "(from broker " << metadata->orig_broker_id() << ":"
+            << metadata->orig_broker_name() << std::endl;
 
   /* Iterate brokers */
   std::cout << " " << metadata->brokers()->size() << " brokers:" << std::endl;
   RdKafka::Metadata::BrokerMetadataIterator ib;
-  for (ib = metadata->brokers()->begin();
-       ib != metadata->brokers()->end();
+  for (ib = metadata->brokers()->begin(); ib != metadata->brokers()->end();
        ++ib) {
-    std::cout << "  broker " << (*ib)->id() << " at "
-              << (*ib)->host() << ":" << (*ib)->port() << std::endl;
+    std::cout << "  broker " << (*ib)->id() << " at " << (*ib)->host() << ":"
+              << (*ib)->port() << std::endl;
   }
   /* Iterate topics */
   std::cout << metadata->topics()->size() << " topics:" << std::endl;
   RdKafka::Metadata::TopicMetadataIterator it;
-  for (it = metadata->topics()->begin();
-       it != metadata->topics()->end();
+  for (it = metadata->topics()->begin(); it != metadata->topics()->end();
        ++it) {
-    std::cout << "  topic \""<< (*it)->topic() << "\" with "
+    std::cout << "  topic \"" << (*it)->topic() << "\" with "
               << (*it)->partitions()->size() << " partitions:";
 
     if ((*it)->err() != RdKafka::ERR_NO_ERROR) {
@@ -87,26 +85,23 @@ static void metadata_print (const std::string &topic,
 
     /* Iterate topic's partitions */
     RdKafka::TopicMetadata::PartitionMetadataIterator ip;
-    for (ip = (*it)->partitions()->begin();
-         ip != (*it)->partitions()->end();
+    for (ip = (*it)->partitions()->begin(); ip != (*it)->partitions()->end();
          ++ip) {
-      std::cout << "    partition " << (*ip)->id()
-                << ", leader " << (*ip)->leader()
-                << ", replicas: ";
+      std::cout << "    partition " << (*ip)->id() << ", leader "
+                << (*ip)->leader() << ", replicas: ";
 
       /* Iterate partition's replicas */
       RdKafka::PartitionMetadata::ReplicasIterator ir;
-      for (ir = (*ip)->replicas()->begin();
-           ir != (*ip)->replicas()->end();
+      for (ir = (*ip)->replicas()->begin(); ir != (*ip)->replicas()->end();
            ++ir) {
-        std::cout << (ir == (*ip)->replicas()->begin() ? "":",") << *ir;
+        std::cout << (ir == (*ip)->replicas()->begin() ? "" : ",") << *ir;
       }
 
       /* Iterate partition's ISRs */
       std::cout << ", isrs: ";
       RdKafka::PartitionMetadata::ISRSIterator iis;
-      for (iis = (*ip)->isrs()->begin(); iis != (*ip)->isrs()->end() ; ++iis)
-        std::cout << (iis == (*ip)->isrs()->begin() ? "":",") << *iis;
+      for (iis = (*ip)->isrs()->begin(); iis != (*ip)->isrs()->end(); ++iis)
+        std::cout << (iis == (*ip)->isrs()->begin() ? "" : ",") << *iis;
 
       if ((*ip)->err() != RdKafka::ERR_NO_ERROR)
         std::cout << ", " << RdKafka::err2str((*ip)->err()) << std::endl;
@@ -117,34 +112,34 @@ static void metadata_print (const std::string &topic,
 }
 
 static volatile sig_atomic_t run = 1;
-static bool exit_eof = false;
+static bool exit_eof             = false;
 
-static void sigterm (int sig) {
+static void sigterm(int sig) {
   run = 0;
 }
 
 
 class ExampleDeliveryReportCb : public RdKafka::DeliveryReportCb {
  public:
-  void dr_cb (RdKafka::Message &message) {
+  void dr_cb(RdKafka::Message &message) {
     std::string status_name;
-    switch (message.status())
-      {
-      case RdKafka::Message::MSG_STATUS_NOT_PERSISTED:
-        status_name = "NotPersisted";
-        break;
-      case RdKafka::Message::MSG_STATUS_POSSIBLY_PERSISTED:
-        status_name = "PossiblyPersisted";
-        break;
-      case RdKafka::Message::MSG_STATUS_PERSISTED:
-        status_name = "Persisted";
-        break;
-      default:
-        status_name = "Unknown?";
-        break;
-      }
-    std::cout << "Message delivery for (" << message.len() << " bytes): " <<
-      status_name << ": " << message.errstr() << std::endl;
+    switch (message.status()) {
+    case RdKafka::Message::MSG_STATUS_NOT_PERSISTED:
+      status_name = "NotPersisted";
+      break;
+    case RdKafka::Message::MSG_STATUS_POSSIBLY_PERSISTED:
+      status_name = "PossiblyPersisted";
+      break;
+    case RdKafka::Message::MSG_STATUS_PERSISTED:
+      status_name = "Persisted";
+      break;
+    default:
+      status_name = "Unknown?";
+      break;
+    }
+    std::cout << "Message delivery for (" << message.len()
+              << " bytes): " << status_name << ": " << message.errstr()
+              << std::endl;
     if (message.key())
       std::cout << "Key: " << *(message.key()) << ";" << std::endl;
   }
@@ -153,32 +148,31 @@ class ExampleDeliveryReportCb : public RdKafka::DeliveryReportCb {
 
 class ExampleEventCb : public RdKafka::EventCb {
  public:
-  void event_cb (RdKafka::Event &event) {
-    switch (event.type())
-    {
-      case RdKafka::Event::EVENT_ERROR:
-        if (event.fatal()) {
-          std::cerr << "FATAL ";
-          run = 0;
-        }
-        std::cerr << "ERROR (" << RdKafka::err2str(event.err()) << "): " <<
-            event.str() << std::endl;
-        break;
+  void event_cb(RdKafka::Event &event) {
+    switch (event.type()) {
+    case RdKafka::Event::EVENT_ERROR:
+      if (event.fatal()) {
+        std::cerr << "FATAL ";
+        run = 0;
+      }
+      std::cerr << "ERROR (" << RdKafka::err2str(event.err())
+                << "): " << event.str() << std::endl;
+      break;
 
-      case RdKafka::Event::EVENT_STATS:
-        std::cerr << "\"STATS\": " << event.str() << std::endl;
-        break;
+    case RdKafka::Event::EVENT_STATS:
+      std::cerr << "\"STATS\": " << event.str() << std::endl;
+      break;
 
-      case RdKafka::Event::EVENT_LOG:
-        fprintf(stderr, "LOG-%i-%s: %s\n",
-                event.severity(), event.fac().c_str(), event.str().c_str());
-        break;
+    case RdKafka::Event::EVENT_LOG:
+      fprintf(stderr, "LOG-%i-%s: %s\n", event.severity(), event.fac().c_str(),
+              event.str().c_str());
+      break;
 
-      default:
-        std::cerr << "EVENT " << event.type() <<
-            " (" << RdKafka::err2str(event.err()) << "): " <<
-            event.str() << std::endl;
-        break;
+    default:
+      std::cerr << "EVENT " << event.type() << " ("
+                << RdKafka::err2str(event.err()) << "): " << event.str()
+                << std::endl;
+      break;
     }
   }
 };
@@ -188,91 +182,91 @@ class ExampleEventCb : public RdKafka::EventCb {
  * in the produce() call. */
 class MyHashPartitionerCb : public RdKafka::PartitionerCb {
  public:
-  int32_t partitioner_cb (const RdKafka::Topic *topic, const std::string *key,
-                          int32_t partition_cnt, void *msg_opaque) {
+  int32_t partitioner_cb(const RdKafka::Topic *topic,
+                         const std::string *key,
+                         int32_t partition_cnt,
+                         void *msg_opaque) {
     return djb_hash(key->c_str(), key->size()) % partition_cnt;
   }
- private:
 
-  static inline unsigned int djb_hash (const char *str, size_t len) {
+ private:
+  static inline unsigned int djb_hash(const char *str, size_t len) {
     unsigned int hash = 5381;
-    for (size_t i = 0 ; i < len ; i++)
+    for (size_t i = 0; i < len; i++)
       hash = ((hash << 5) + hash) + str[i];
     return hash;
   }
 };
 
-void msg_consume(RdKafka::Message* message, void* opaque) {
+void msg_consume(RdKafka::Message *message, void *opaque) {
   const RdKafka::Headers *headers;
 
   switch (message->err()) {
-    case RdKafka::ERR__TIMED_OUT:
-      break;
+  case RdKafka::ERR__TIMED_OUT:
+    break;
 
-    case RdKafka::ERR_NO_ERROR:
-      /* Real message */
-      std::cout << "Read msg at offset " << message->offset() << std::endl;
-      if (message->key()) {
-        std::cout << "Key: " << *message->key() << std::endl;
+  case RdKafka::ERR_NO_ERROR:
+    /* Real message */
+    std::cout << "Read msg at offset " << message->offset() << std::endl;
+    if (message->key()) {
+      std::cout << "Key: " << *message->key() << std::endl;
+    }
+    headers = message->headers();
+    if (headers) {
+      std::vector<RdKafka::Headers::Header> hdrs = headers->get_all();
+      for (size_t i = 0; i < hdrs.size(); i++) {
+        const RdKafka::Headers::Header hdr = hdrs[i];
+
+        if (hdr.value() != NULL)
+          printf(" Header: %s = \"%.*s\"\n", hdr.key().c_str(),
+                 (int)hdr.value_size(), (const char *)hdr.value());
+        else
+          printf(" Header:  %s = NULL\n", hdr.key().c_str());
       }
-      headers = message->headers();
-      if (headers) {
-        std::vector<RdKafka::Headers::Header> hdrs = headers->get_all();
-        for (size_t i = 0 ; i < hdrs.size() ; i++) {
-          const RdKafka::Headers::Header hdr = hdrs[i];
+    }
+    printf("%.*s\n", static_cast<int>(message->len()),
+           static_cast<const char *>(message->payload()));
+    break;
 
-          if (hdr.value() != NULL)
-            printf(" Header: %s = \"%.*s\"\n",
-                   hdr.key().c_str(),
-                   (int)hdr.value_size(), (const char *)hdr.value());
-          else
-            printf(" Header:  %s = NULL\n", hdr.key().c_str());
-        }
-      }
-      printf("%.*s\n",
-        static_cast<int>(message->len()),
-        static_cast<const char *>(message->payload()));
-      break;
-
-    case RdKafka::ERR__PARTITION_EOF:
-      /* Last message */
-      if (exit_eof) {
-        run = 0;
-      }
-      break;
-
-    case RdKafka::ERR__UNKNOWN_TOPIC:
-    case RdKafka::ERR__UNKNOWN_PARTITION:
-      std::cerr << "Consume failed: " << message->errstr() << std::endl;
+  case RdKafka::ERR__PARTITION_EOF:
+    /* Last message */
+    if (exit_eof) {
       run = 0;
-      break;
+    }
+    break;
 
-    default:
-      /* Errors */
-      std::cerr << "Consume failed: " << message->errstr() << std::endl;
-      run = 0;
+  case RdKafka::ERR__UNKNOWN_TOPIC:
+  case RdKafka::ERR__UNKNOWN_PARTITION:
+    std::cerr << "Consume failed: " << message->errstr() << std::endl;
+    run = 0;
+    break;
+
+  default:
+    /* Errors */
+    std::cerr << "Consume failed: " << message->errstr() << std::endl;
+    run = 0;
   }
 }
 
 
 class ExampleConsumeCb : public RdKafka::ConsumeCb {
  public:
-  void consume_cb (RdKafka::Message &msg, void *opaque) {
+  void consume_cb(RdKafka::Message &msg, void *opaque) {
     msg_consume(&msg, opaque);
   }
 };
 
 
 
-int main (int argc, char **argv) {
+int main(int argc, char **argv) {
   std::string brokers = "localhost";
   std::string errstr;
   std::string topic_str;
   std::string mode;
   std::string debug;
-  int32_t partition = RdKafka::Topic::PARTITION_UA;
+  int32_t partition    = RdKafka::Topic::PARTITION_UA;
   int64_t start_offset = RdKafka::Topic::OFFSET_BEGINNING;
-  bool do_conf_dump = false;
+  bool do_conf_dump    = false;
   int opt;
   MyHashPartitionerCb hash_partitioner;
   int use_ccb = 0;
@@ -280,7 +274,7 @@ int main (int argc, char **argv) {
   /*
    * Create configuration objects
    */
-  RdKafka::Conf *conf = RdKafka::Conf::create(RdKafka::Conf::CONF_GLOBAL);
+  RdKafka::Conf *conf  = RdKafka::Conf::create(RdKafka::Conf::CONF_GLOBAL);
   RdKafka::Conf *tconf = RdKafka::Conf::create(RdKafka::Conf::CONF_TOPIC);
 
 
@@ -311,20 +305,20 @@ int main (int argc, char **argv) {
       break;
     case 'z':
       if (conf->set("compression.codec", optarg, errstr) !=
-	  RdKafka::Conf::CONF_OK) {
-	std::cerr << errstr << std::endl;
-	exit(1);
+          RdKafka::Conf::CONF_OK) {
+        std::cerr << errstr << std::endl;
+        exit(1);
       }
       break;
     case 'o':
       if (!strcmp(optarg, "end"))
-	start_offset = RdKafka::Topic::OFFSET_END;
+        start_offset = RdKafka::Topic::OFFSET_END;
       else if (!strcmp(optarg, "beginning"))
-	start_offset = RdKafka::Topic::OFFSET_BEGINNING;
+        start_offset = RdKafka::Topic::OFFSET_BEGINNING;
       else if (!strcmp(optarg, "stored"))
-	start_offset = RdKafka::Topic::OFFSET_STORED;
+        start_offset = RdKafka::Topic::OFFSET_STORED;
       else
-	start_offset = strtoll(optarg, NULL, 10);
+        start_offset = strtoll(optarg, NULL, 10);
       break;
     case 'e':
       exit_eof = true;
@@ -339,50 +333,47 @@ int main (int argc, char **argv) {
         exit(1);
       }
       break;
-    case 'X':
-      {
-	char *name, *val;
+    case 'X': {
+      char *name, *val;
 
-	if (!strcmp(optarg, "dump")) {
-	  do_conf_dump = true;
-	  continue;
-	}
+      if (!strcmp(optarg, "dump")) {
+        do_conf_dump = true;
+        continue;
+      }
 
-	name = optarg;
-	if (!(val = strchr(name, '='))) {
-          std::cerr << "%% Expected -X property=value, not " <<
-              name << std::endl;
-	  exit(1);
-	}
+      name = optarg;
+      if (!(val = strchr(name, '='))) {
+        std::cerr << "%% Expected -X property=value, not " << name << std::endl;
+        exit(1);
+      }
 
-	*val = '\0';
-	val++;
+      *val = '\0';
+      val++;
 
-	/* Try "topic." prefixed properties on topic
-	 * conf first, and then fall through to global if
-	 * it didnt match a topic configuration property. */
-        RdKafka::Conf::ConfResult res;
-	if (!strncmp(name, "topic.", strlen("topic.")))
-          res = tconf->set(name+strlen("topic."), val, errstr);
-        else
-	  res = conf->set(name, val, errstr);
+      /* Try "topic." prefixed properties on topic
+       * conf first, and then fall through to global if
+       * it didnt match a topic configuration property. */
+      RdKafka::Conf::ConfResult res;
+      if (!strncmp(name, "topic.", strlen("topic.")))
+        res = tconf->set(name + strlen("topic."), val, errstr);
+      else
+        res = conf->set(name, val, errstr);
 
-	if (res != RdKafka::Conf::CONF_OK) {
-          std::cerr << errstr << std::endl;
-	  exit(1);
-	}
+      if (res != RdKafka::Conf::CONF_OK) {
+        std::cerr << errstr << std::endl;
+        exit(1);
+      }
+    } break;
+
+    case 'f':
+      if (!strcmp(optarg, "ccb"))
+        use_ccb = 1;
+      else {
+        std::cerr << "Unknown option: " << optarg << std::endl;
+        exit(1);
       }
       break;
 
-      case 'f':
-        if (!strcmp(optarg, "ccb"))
-          use_ccb = 1;
-        else {
-          std::cerr << "Unknown option: " << optarg << std::endl;
-          exit(1);
-        }
-        break;
-
     default:
       goto usage;
     }
@@ -390,8 +381,8 @@ int main (int argc, char **argv) {
 
   if (mode.empty() || (topic_str.empty() && mode != "L") || optind != argc) {
   usage:
-	  std::string features;
-	  conf->get("builtin.features", features);
+    std::string features;
+    conf->get("builtin.features", features);
     fprintf(stderr,
             "Usage: %s [-C|-P] -t <topic> "
             "[-p <partition>] [-b <host1:port1,host2:port2,..>]\n"
@@ -430,11 +421,9 @@ int main (int argc, char **argv) {
             "\n"
             "\n"
             "\n",
-	    argv[0],
-	    RdKafka::version_str().c_str(), RdKafka::version(),
-		features.c_str(),
-	    RdKafka::get_debug_contexts().c_str());
-	exit(1);
+            argv[0], RdKafka::version_str().c_str(), RdKafka::version(),
+            features.c_str(), RdKafka::get_debug_contexts().c_str());
+    exit(1);
   }
 
 
@@ -456,7 +445,7 @@ int main (int argc, char **argv) {
   if (do_conf_dump) {
     int pass;
 
-    for (pass = 0 ; pass < 2 ; pass++) {
+    for (pass = 0; pass < 2; pass++) {
       std::list<std::string> *dump;
       if (pass == 0) {
         dump = conf->dump();
@@ -467,7 +456,7 @@ int main (int argc, char **argv) {
       }
 
       for (std::list<std::string>::iterator it = dump->begin();
-           it != dump->end(); ) {
+           it != dump->end();) {
         std::cout << *it << " = ";
         it++;
         std::cout << *it << std::endl;
@@ -487,7 +476,7 @@ int main (int argc, char **argv) {
      * Producer mode
      */
 
-    if(topic_str.empty())
+    if (topic_str.empty())
       goto usage;
 
     ExampleDeliveryReportCb ex_dr_cb;
@@ -515,7 +504,7 @@ int main (int argc, char **argv) {
     for (std::string line; run && std::getline(std::cin, line);) {
       if (line.empty()) {
         producer->poll(0);
-	continue;
+        continue;
       }
 
       RdKafka::Headers *headers = RdKafka::Headers::create();
@@ -526,27 +515,27 @@ int main (int argc, char **argv) {
        * Produce message
        */
       RdKafka::ErrorCode resp =
-        producer->produce(topic_str, partition,
-                          RdKafka::Producer::RK_MSG_COPY /* Copy payload */,
-                          /* Value */
-                          const_cast<char *>(line.c_str()), line.size(),
-                          /* Key */
-                          NULL, 0,
-                          /* Timestamp (defaults to now) */
-                          0,
-                          /* Message headers, if any */
-                          headers,
-                          /* Per-message opaque value passed to
-                           * delivery report */
-                          NULL);
+          producer->produce(topic_str, partition,
+                            RdKafka::Producer::RK_MSG_COPY /* Copy payload */,
+                            /* Value */
+                            const_cast<char *>(line.c_str()), line.size(),
+                            /* Key */
+                            NULL, 0,
+                            /* Timestamp (defaults to now) */
+                            0,
+                            /* Message headers, if any */
+                            headers,
+                            /* Per-message opaque value passed to
+                             * delivery report */
+                            NULL);
       if (resp != RdKafka::ERR_NO_ERROR) {
-        std::cerr << "% Produce failed: " <<
-          RdKafka::err2str(resp) << std::endl;
+        std::cerr << "% Produce failed: " << RdKafka::err2str(resp)
+                  << std::endl;
         delete headers; /* Headers are automatically deleted on produce()
                          * success. */
       } else {
-        std::cerr << "% Produced message (" << line.size() << " bytes)" <<
-          std::endl;
+        std::cerr << "% Produced message (" << line.size() << " bytes)"
+                  << std::endl;
       }
 
       producer->poll(0);
@@ -568,7 +557,7 @@ int main (int argc, char **argv) {
 
     conf->set("enable.partition.eof", "true", errstr);
 
-    if(topic_str.empty())
+    if (topic_str.empty())
       goto usage;
 
     /*
@@ -585,8 +574,8 @@ int main (int argc, char **argv) {
     /*
      * Create topic handle.
      */
-    RdKafka::Topic *topic = RdKafka::Topic::create(consumer, topic_str,
-						   tconf, errstr);
+    RdKafka::Topic *topic =
+        RdKafka::Topic::create(consumer, topic_str, tconf, errstr);
     if (!topic) {
       std::cerr << "Failed to create topic: " << errstr << std::endl;
       exit(1);
@@ -597,8 +586,8 @@ int main (int argc, char **argv) {
      */
     RdKafka::ErrorCode resp = consumer->start(topic, partition, start_offset);
     if (resp != RdKafka::ERR_NO_ERROR) {
-      std::cerr << "Failed to start consumer: " <<
-	RdKafka::err2str(resp) << std::endl;
+      std::cerr << "Failed to start consumer: " << RdKafka::err2str(resp)
+                << std::endl;
       exit(1);
     }
 
@@ -609,8 +598,8 @@ int main (int argc, char **argv) {
      */
     while (run) {
       if (use_ccb) {
-        consumer->consume_callback(topic, partition, 1000,
-                                   &ex_consume_cb, &use_ccb);
+        consumer->consume_callback(topic, partition, 1000, &ex_consume_cb,
+                                   &use_ccb);
       } else {
         RdKafka::Message *msg = consumer->consume(topic, partition, 1000);
         msg_consume(msg, NULL);
@@ -646,7 +635,7 @@ int main (int argc, char **argv) {
      * Create topic handle.
      */
     RdKafka::Topic *topic = NULL;
-    if(!topic_str.empty()) {
+    if (!topic_str.empty()) {
       topic = RdKafka::Topic::create(producer, topic_str, tconf, errstr);
       if (!topic) {
         std::cerr << "Failed to create topic: " << errstr << std::endl;
@@ -658,13 +647,13 @@ int main (int argc, char **argv) {
       class RdKafka::Metadata *metadata;
 
       /* Fetch metadata */
-      RdKafka::ErrorCode err = producer->metadata(!topic, topic,
-                              &metadata, 5000);
+      RdKafka::ErrorCode err =
+          producer->metadata(!topic, topic, &metadata, 5000);
       if (err != RdKafka::ERR_NO_ERROR) {
-        std::cerr << "%% Failed to acquire metadata: "
-                  << RdKafka::err2str(err) << std::endl;
-              run = 0;
-              break;
+        std::cerr << "%% Failed to acquire metadata: " << RdKafka::err2str(err)
+                  << std::endl;
+        run = 0;
+        break;
       }
 
       metadata_print(topic_str, metadata);
@@ -672,7 +661,6 @@ int main (int argc, char **argv) {
       delete metadata;
       run = 0;
     }
-
   }
 
   delete conf;
