@@ -11,27 +11,60 @@ import sys
 import argparse
 import time
 import packaging
+import nugetpackage
+import staticpackage
 
 
 dry_run = False
 
 
-
 if __name__ == '__main__':
 
     parser = argparse.ArgumentParser()
-    parser.add_argument("--no-s3", help="Don't collect from S3", action="store_true")
+    parser.add_argument(
+        "--s3",
+        help="Collect artifacts from S3 bucket",
+        action="store_true")
     parser.add_argument("--dry-run",
-                        help="Locate artifacts but don't actually download or do anything",
+                        help="Locate artifacts but don't actually "
+                        "download or do anything",
                         action="store_true")
-    parser.add_argument("--directory", help="Download directory (default: dl-<tag>)", default=None)
-    parser.add_argument("--no-cleanup", help="Don't clean up temporary folders", action="store_true")
-    parser.add_argument("--sha", help="Also match on this git sha1", default=None)
-    parser.add_argument("--nuget-version", help="The nuget package version (defaults to same as tag)", default=None)
-    parser.add_argument("--upload", help="Upload package to after building, using provided NuGet API key (either file or the key itself)", default=None,
+    parser.add_argument(
+        "--directory",
+        help="Download directory (default: dl-<tag>)",
+        default=None)
+    parser.add_argument(
+        "--no-cleanup",
+        help="Don't clean up temporary folders",
+        action="store_true")
+    parser.add_argument(
+        "--sha",
+        help="Also match on this git sha1",
+        default=None)
+    parser.add_argument(
+        "--ignore-tag",
+        help="Ignore the artifacts' tag attribute (for devel use only)",
+        action="store_true",
+        default=False)
+    parser.add_argument(
+        "--nuget-version",
+        help="The nuget package version (defaults to same as tag)",
+        default=None)
+    parser.add_argument("--upload", help="Upload package to after building, "
+                        "using provided NuGet API key "
+                        "(either file or the key itself)",
+                        default=None,
                         type=str)
-    parser.add_argument("--class", help="Packaging class (see packaging.py)", default="NugetPackage", dest="pkgclass")
-    parser.add_argument("--retries", help="Number of retries to collect artifacts", default=0, type=int)
+    parser.add_argument(
+        "--class",
+        help="Packaging class (either NugetPackage or StaticPackage)",
+        default="NugetPackage",
+        dest="pkgclass")
+    parser.add_argument(
+        "--retries",
+        help="Number of retries to collect artifacts",
+        default=0,
+        type=int)
     parser.add_argument("tag", help="Git tag to collect")
 
     args = parser.parse_args()
@@ -40,15 +73,24 @@ if __name__ == '__main__':
     if not args.directory:
         args.directory = 'dl-%s' % args.tag
 
-    match = {'tag': args.tag}
+    match = {}
+    if not args.ignore_tag:
+        match['tag'] = args.tag
+
     if args.sha is not None:
         match['sha'] = args.sha
 
-    pkgclass = getattr(packaging, args.pkgclass)
+    if args.pkgclass == "NugetPackage":
+        pkgclass = nugetpackage.NugetPackage
+    elif args.pkgclass == "StaticPackage":
+        pkgclass = staticpackage.StaticPackage
+    else:
+        raise ValueError(f'Unknown packaging class {args.pkgclass}: '
+                         'should be one of NugetPackage or StaticPackage')
 
     try:
         match.update(getattr(pkgclass, 'match'))
-    except:
+    except BaseException:
         pass
 
     arts = packaging.Artifacts(match, args.directory)
@@ -57,10 +99,10 @@ if __name__ == '__main__':
     arts.collect_local('common', req_tag=False)
 
     while True:
-        if not args.no_s3:
+        if args.s3:
             arts.collect_s3()
-        else:
-            arts.collect_local(arts.dlpath)
+
+        arts.collect_local(arts.dlpath)
 
         if len(arts.artifacts) == 0:
             raise ValueError('No artifacts found for %s' % match)
@@ -70,9 +112,10 @@ if __name__ == '__main__':
             print(' %s' % a.lpath)
         print('')
 
-        package_version = match['tag']
         if args.nuget_version is not None:
             package_version = args.nuget_version
+        else:
+            package_version = args.tag
 
         print('')
 
@@ -86,7 +129,7 @@ if __name__ == '__main__':
             pkgfile = p.build(buildtype='release')
             break
         except packaging.MissingArtifactError as e:
-            if retries <= 0 or args.no_s3:
+            if retries <= 0 or not args.s3:
                 if not args.no_cleanup:
                     p.cleanup()
                 raise e
@@ -119,5 +162,6 @@ if __name__ == '__main__':
 
         print('Uploading %s to NuGet' % pkgfile)
         r = os.system("./push-to-nuget.sh '%s' %s" % (nuget_key, pkgfile))
-        assert int(r) == 0, "NuGet upload failed with exit code {}, see previous errors".format(r)
+        assert int(r) == 0, \
+            f"NuGet upload failed with exit code {r}, see previous errors"
         print('%s successfully uploaded to NuGet' % pkgfile)
