@@ -375,8 +375,10 @@ struct rd_kafka_buf_s { /* rd_kafka_buf_t */
 
         union {
                 struct {
-                        rd_list_t *topics;     /* Requested topics (char *) */
-                        char *reason;          /* Textual reason */
+                        rd_list_t *topics; /* Requested topics (char *) */
+                        rd_list_t *
+                            topic_ids; /* Requested topic ids rd_kafka_Uuid_t */
+                        char *reason;  /* Textual reason */
                         rd_kafka_op_t *rko;    /* Originating rko with replyq
                                                 * (if any) */
                         rd_bool_t all_topics;  /**< Full/All topics requested */
@@ -717,15 +719,29 @@ struct rd_kafka_buf_s { /* rd_kafka_buf_t */
         } while (0)
 
 /**
- * Skip a string.
+ * Skip a string without flexver.
  */
-#define rd_kafka_buf_skip_str(rkbuf)                                           \
+#define rd_kafka_buf_skip_str_no_flexver(rkbuf)                                \
         do {                                                                   \
                 int16_t _slen;                                                 \
                 rd_kafka_buf_read_i16(rkbuf, &_slen);                          \
                 rd_kafka_buf_skip(rkbuf, RD_KAFKAP_STR_LEN0(_slen));           \
         } while (0)
 
+/**
+ * Skip a string (generic).
+ */
+#define rd_kafka_buf_skip_str(rkbuf)                                           \
+        do {                                                                   \
+                if ((rkbuf)->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER) {            \
+                        uint64_t _uva;                                         \
+                        rd_kafka_buf_read_uvarint(rkbuf, &_uva);               \
+                        rd_kafka_buf_skip(                                     \
+                            rkbuf, RD_KAFKAP_STR_LEN0(((int64_t)_uva) - 1));   \
+                } else {                                                       \
+                        rd_kafka_buf_skip_str_no_flexver(rkbuf);               \
+                }                                                              \
+        } while (0)
 /**
  * Read Kafka COMPACT_BYTES representation (VARINT+N) or
  * standard BYTES representation(4+N).
@@ -821,11 +837,56 @@ struct rd_kafka_buf_s { /* rd_kafka_buf_t */
         } while (0)
 
 /**
- * @brief Write tags at the current position in the buffer.
- * @remark Currently always writes empty tags.
- * @remark Change to ..write_uvarint() when actual tags are supported.
+ * @brief Read KIP-482 Tags at current position in the buffer using
+ *        the `read_tag` function receiving the `opaque' pointer.
  */
-#define rd_kafka_buf_write_tags(rkbuf)                                         \
+#define rd_kafka_buf_read_tags(rkbuf, read_tag, ...)                           \
+        do {                                                                   \
+                uint64_t _tagcnt;                                              \
+                if (!((rkbuf)->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER))           \
+                        break;                                                 \
+                rd_kafka_buf_read_uvarint(rkbuf, &_tagcnt);                    \
+                while (_tagcnt-- > 0) {                                        \
+                        uint64_t _tagtype, _taglen;                            \
+                        rd_kafka_buf_read_uvarint(rkbuf, &_tagtype);           \
+                        rd_kafka_buf_read_uvarint(rkbuf, &_taglen);            \
+                        int _read_tag_resp =                                   \
+                            read_tag(rkbuf, _tagtype, _taglen, __VA_ARGS__);   \
+                        if (_read_tag_resp == -1)                              \
+                                goto err_parse;                                \
+                        if (!_read_tag_resp && _taglen > 0)                    \
+                                rd_kafka_buf_skip(rkbuf, (size_t)(_taglen));   \
+                }                                                              \
+        } while (0)
+
+/**
+ * @brief Write \p tagcnt tags at the current position in the buffer.
+ * Calling \p write_tag to write each one with \p rkbuf , tagtype
+ * argument and the remaining arguments.
+ */
+#define rd_kafka_buf_write_tags(rkbuf, write_tag, tags, tagcnt, ...)           \
+        do {                                                                   \
+                uint64_t i;                                                    \
+                if (!((rkbuf)->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER))           \
+                        break;                                                 \
+                rd_kafka_buf_write_uvarint(rkbuf, tagcnt);                     \
+                for (i = 0; i < tagcnt; i++) {                                 \
+                        size_t of_taglen, prev_buf_len;                        \
+                        rd_kafka_buf_write_uvarint(rkbuf, tags[i]);            \
+                        of_taglen    = rd_kafka_buf_write_arraycnt_pos(rkbuf); \
+                        prev_buf_len = (rkbuf)->rkbuf_buf.rbuf_len;            \
+                        write_tag(rkbuf, tags[i], __VA_ARGS__);                \
+                        rd_kafka_buf_finalize_arraycnt(                        \
+                            rkbuf, of_taglen,                                  \
+                            (rkbuf)->rkbuf_buf.rbuf_len - prev_buf_len - 1);   \
+                }                                                              \
+        } while (0)
+
+
+/**
+ * @brief Write empty tags at the current position in the buffer.
+ */
+#define rd_kafka_buf_write_tags_empty(rkbuf)                                   \
         do {                                                                   \
                 if (!((rkbuf)->rkbuf_flags & RD_KAFKA_OP_F_FLEXVER))           \
                         break;                                                 \
