@@ -65,6 +65,26 @@
 #define RD_KAFKA_MSGSET_V2_ATTR_TRANSACTIONAL (1 << 4)
 #define RD_KAFKA_MSGSET_V2_ATTR_CONTROL       (1 << 5)
 
+/**
+ * @struct Error data for a batch index that caused the batch to be dropped.
+ */
+typedef struct rd_kafka_Produce_result_record_error {
+        int64_t batch_index; /**< Batch index */
+        char *errstr;        /**< Error message for batch_index */
+} rd_kafka_Produce_result_record_error_t;
+
+/**
+ * @struct Result and return values from ProduceResponse
+ */
+typedef struct rd_kafka_Produce_result {
+        int64_t offset;    /**< Assigned offset of first message */
+        int64_t timestamp; /**< (Possibly assigned) offset of first message */
+        char *errstr;      /**< Common error message */
+        rd_kafka_Produce_result_record_error_t
+            *record_errors; /**< Errors for records that caused the batch to be
+                               dropped */
+        int32_t record_errors_cnt; /**< record_errors count */
+} rd_kafka_Produce_result_t;
 
 typedef struct rd_kafka_msg_s {
         rd_kafka_message_t rkm_rkmessage; /* MUST be first field */
@@ -122,6 +142,7 @@ typedef struct rd_kafka_msg_s {
                                               *   identically reconstructed.
                                               */
                         int retries;         /* Number of retries so far */
+                        const char *errstr;  /* Error string for this message */
                 } producer;
 #define rkm_ts_timeout rkm_u.producer.ts_timeout
 #define rkm_ts_enq     rkm_u.producer.ts_enq
@@ -576,6 +597,16 @@ static RD_INLINE RD_UNUSED int32_t rd_kafka_seq_wrap(int64_t seq) {
 
 void rd_kafka_msgq_dump(FILE *fp, const char *what, rd_kafka_msgq_t *rkmq);
 
+rd_kafka_Produce_result_t *rd_kafka_Produce_result_new(int64_t offset,
+                                                       int64_t timestamp);
+
+void rd_kafka_Produce_result_destroy(rd_kafka_Produce_result_t *result);
+
+rd_kafka_Produce_result_t *
+rd_kafka_Produce_result_copy(const rd_kafka_Produce_result_t *result);
+
+/* Unit tests */
+
 rd_kafka_msg_t *ut_rd_kafka_msg_new(size_t msgsize);
 void ut_rd_kafka_msgq_purge(rd_kafka_msgq_t *rkmq);
 int unittest_msg(void);
