@@ -62,9 +62,7 @@ static void consume_pause(void) {
         test_conf_set(conf, "enable.partition.eof", "true");
         test_topic_conf_set(tconf, "auto.offset.reset", "smallest");
 
-        test_create_topic(NULL, topic, partition_cnt, 1);
-
-        test_wait_topic_exists(NULL, topic, 10 * 1000);
+        test_create_topic_wait_exists(NULL, topic, partition_cnt, 1, 10 * 1000);
 
         /* Produce messages */
         testid =
@@ -260,9 +258,8 @@ static void consume_pause_resume_after_reassign(void) {
 
         test_conf_init(&conf, NULL, 60);
 
-        test_create_topic(NULL, topic, (int)partition + 1, 1);
-
-        test_wait_topic_exists(NULL, topic, 10 * 1000);
+        test_create_topic_wait_exists(NULL, topic, (int)partition + 1, 1,
+                                      10 * 1000);
 
         /* Produce messages */
         testid = test_produce_msgs_easy(topic, 0, partition, msgcnt);
@@ -419,9 +416,8 @@ static void consume_subscribe_assign_pause_resume(void) {
 
         test_conf_init(&conf, NULL, 20);
 
-        test_create_topic(NULL, topic, (int)partition + 1, 1);
-
-        test_wait_topic_exists(NULL, topic, 10 * 1000);
+        test_create_topic_wait_exists(NULL, topic, (int)partition + 1, 1,
+                                      10 * 1000);
 
         /* Produce messages */
         testid = test_produce_msgs_easy(topic, 0, partition, msgcnt);
@@ -472,9 +468,8 @@ static void consume_seek_pause_resume(void) {
 
         test_conf_init(&conf, NULL, 20);
 
-        test_create_topic(NULL, topic, (int)partition + 1, 1);
-
-        test_wait_topic_exists(NULL, topic, 10 * 1000);
+        test_create_topic_wait_exists(NULL, topic, (int)partition + 1, 1,
+                                      10 * 1000);
 
         /* Produce messages */
         testid = test_produce_msgs_easy(topic, 0, partition, msgcnt);
