@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2019, Magnus Edenhill
+ * Copyright (c) 2019-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,6 +28,7 @@
 
 #include "test.h"
 
+#include "../src/rdkafka_proto.h"
 
 /**
  * @name KafkaConsumer static membership tests
@@ -160,7 +161,7 @@ static void do_test_static_group_rebalance(void) {
         c[0].mv = &mv;
         c[1].mv = &mv;
 
-        test_create_topic(NULL, topic, 3, 1);
+        test_create_topic_wait_exists(NULL, topic, 3, 1, 5000);
         test_produce_msgs_easy(topic, testid, RD_KAFKA_PARTITION_UA, msgcnt);
 
         test_conf_set(conf, "max.poll.interval.ms", "9000");
@@ -255,7 +256,8 @@ static void do_test_static_group_rebalance(void) {
          * New topics matching the subscription pattern should cause
          * group rebalance
          */
-        test_create_topic(c->rk, tsprintf("%snew", topic), 1, 1);
+        test_create_topic_wait_exists(c->rk, tsprintf("%snew", topic), 1, 1,
+                                      5000);
 
         /* Await revocation */
         rebalance_start        = test_clock();
@@ -440,9 +442,9 @@ is_fatal_cb(rd_kafka_t *rk, rd_kafka_resp_err_t err, const char *reason) {
 }
 
 /**
- * @brief Test that consumer fencing raises a fatal error
+ * @brief Test that consumer fencing raises a fatal error, classic protocol
  */
-static void do_test_fenced_member(void) {
+static void do_test_fenced_member_classic(void) {
         rd_kafka_t *c[3]; /* 0: consumer2b, 1: consumer1, 2: consumer2a */
         rd_kafka_conf_t *conf;
         const char *topic =
@@ -458,9 +460,11 @@ static void do_test_fenced_member(void) {
         test_create_topic(NULL, topic, 3, 1);
 
         test_conf_set(conf, "group.instance.id", "consumer1");
+        test_conf_set(conf, "client.id", "consumer1");
         c[1] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
 
         test_conf_set(conf, "group.instance.id", "consumer2");
+        test_conf_set(conf, "client.id", "consumer2a");
         c[2] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
 
         test_wait_topic_exists(c[2], topic, 5000);
@@ -473,6 +477,7 @@ static void do_test_fenced_member(void) {
         /* Create conflicting consumer */
         TEST_SAY("Creating conflicting consumer2 instance\n");
         test_conf_set(conf, "group.instance.id", "consumer2");
+        test_conf_set(conf, "client.id", "consumer2b");
         c[0] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
         rd_kafka_conf_destroy(conf);
 
@@ -523,13 +528,274 @@ static void do_test_fenced_member(void) {
         SUB_TEST_PASS();
 }
 
+/**
+ * @brief Test that consumer fencing raises a fatal error,
+ *        consumer protocol (KIP-848).
+ *        The difference with the behavior of the classic one is that
+ *        the member that is fenced is the one that is joining the group
+ *        and not the one that was already in the group.
+ *        Also the error is ERR_UNRELEASED_INSTANCE_ID instead of
+ *        ERR_FENCED_INSTANCE_ID.
+ */
+static void do_test_fenced_member_consumer(void) {
+        rd_kafka_t *c[3]; /* 0: consumer2b, 1: consumer1, 2: consumer2a */
+        rd_kafka_conf_t *conf;
+        const char *topic =
+            test_mk_topic_name("0102_static_group_rebalance", 1);
+        rd_kafka_message_t *rkm;
+        char errstr[512];
+        rd_kafka_resp_err_t err;
 
+        SUB_TEST();
+
+        test_conf_init(&conf, NULL, 30);
+
+        test_create_topic(NULL, topic, 3, 1);
+
+        test_conf_set(conf, "group.instance.id", "consumer1");
+        test_conf_set(conf, "client.id", "consumer1");
+        c[1] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
+
+        test_conf_set(conf, "group.instance.id", "consumer2");
+        test_conf_set(conf, "client.id", "consumer2a");
+        c[2] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
+
+        test_wait_topic_exists(c[2], topic, 5000);
+
+        test_consumer_subscribe(c[1], topic);
+        test_consumer_subscribe(c[2], topic);
+
+        await_assignment_multi("Awaiting initial assignments", &c[1], 2);
+
+        /* Create conflicting consumer */
+        TEST_SAY("Creating conflicting consumer2 instance\n");
+        test_conf_set(conf, "group.instance.id", "consumer2");
+        test_conf_set(conf, "client.id", "consumer2b");
+        c[0] = test_create_consumer(topic, NULL, rd_kafka_conf_dup(conf), NULL);
+        rd_kafka_conf_destroy(conf);
+
+        test_curr->is_fatal_cb = is_fatal_cb;
+        valid_fatal_rk = c[0]; /* consumer2b is the consumer that should fail */
+
+        test_consumer_subscribe(c[0], topic);
+
+        /* consumer1 should not be affected (other than a rebalance which
+         * we ignore here)... */
+        test_consumer_poll_no_msgs("consumer1", c[1], 0, 5000);
+
+        /* consumer2b should be fenced off on joining */
+        rkm = rd_kafka_consumer_poll(c[0], 5000);
+        TEST_ASSERT(rkm != NULL, "Expected error, not timeout");
+        TEST_ASSERT(rkm->err == RD_KAFKA_RESP_ERR__FATAL,
+                    "Expected ERR__FATAL, not %s: %s",
+                    rd_kafka_err2str(rkm->err), rd_kafka_message_errstr(rkm));
+        TEST_SAY("Fenced consumer returned expected: %s: %s\n",
+                 rd_kafka_err2name(rkm->err), rd_kafka_message_errstr(rkm));
+        rd_kafka_message_destroy(rkm);
+
+
+        /* Read the actual error */
+        err = rd_kafka_fatal_error(c[0], errstr, sizeof(errstr));
+        TEST_SAY("%s fatal error: %s: %s\n", rd_kafka_name(c[0]),
+                 rd_kafka_err2name(err), errstr);
+        TEST_ASSERT(
+            err == RD_KAFKA_RESP_ERR_UNRELEASED_INSTANCE_ID,
+            "Expected ERR_UNRELEASED_INSTANCE_ID as fatal error, not %s",
+            rd_kafka_err2name(err));
+
+        TEST_SAY("close\n");
+        /* Close consumer2b, should also return a fatal error */
+        err = rd_kafka_consumer_close(c[0]);
+        TEST_ASSERT(err == RD_KAFKA_RESP_ERR__FATAL,
+                    "Expected close on %s to return ERR__FATAL, not %s",
+                    rd_kafka_name(c[0]), rd_kafka_err2name(err));
+
+        rd_kafka_destroy(c[0]);
+
+        /* consumer1 and consumer2a should be fine and get their
+         * assignments */
+        await_assignment_multi("Awaiting post-fencing assignment", &c[1], 2);
+
+        rd_kafka_destroy(c[1]);
+        rd_kafka_destroy(c[2]);
+
+        SUB_TEST_PASS();
+}
+/**
+ * @brief Create a new consumer with given \p boostraps
+ *        \p group_id and \p group_instance_id .
+ */
+static rd_kafka_t *create_consumer(const char *bootstraps,
+                                   const char *group_id,
+                                   const char *group_instance_id) {
+        rd_kafka_conf_t *conf;
+        test_conf_init(&conf, NULL, 0);
+        test_conf_set(conf, "bootstrap.servers", bootstraps);
+        test_conf_set(conf, "group.instance.id", group_instance_id);
+        test_conf_set(conf, "auto.offset.reset", "earliest");
+        test_conf_set(conf, "enable.partition.eof", "true");
+        return test_create_consumer(group_id, NULL, conf, NULL);
+}
+
+/**
+ * @brief Get generation id of consumer \p consumer .
+ */
+static int32_t consumer_generation_id(rd_kafka_t *consumer) {
+        rd_kafka_consumer_group_metadata_t *group_metadata;
+        int32_t generation_id;
+
+        group_metadata = rd_kafka_consumer_group_metadata(consumer);
+        generation_id =
+            rd_kafka_consumer_group_metadata_generation_id(group_metadata);
+        rd_kafka_consumer_group_metadata_destroy(group_metadata);
+        return generation_id;
+}
+
+/**
+ * @brief Check if the API key in \p request is the same as that
+ *        pointed by \p opaque .
+ */
+static rd_bool_t is_api_key(rd_kafka_mock_request_t *request, void *opaque) {
+        int32_t api_key = *(int32_t *)opaque;
+        return rd_kafka_mock_request_api_key(request) == api_key;
+}
+
+/**
+ * @brief Static group membership tests with the mock cluster.
+ *        Checks that consumer returns the same assignment
+ *        and generation id after re-joining.
+ *
+ *        variation 1) consumer 1 leaves and rejoins the group.
+ *        variation 2) consumer 1 leaves and a new consumer with same
+ *                     group.instance.id joins the group.
+ */
+static void do_test_static_membership_mock(int variation) {
+        const char *bootstraps;
+        rd_kafka_mock_cluster_t *mcluster;
+        int32_t api_key   = RD_KAFKAP_ConsumerGroupHeartbeat;
+        const char *topic = test_mk_topic_name(__FUNCTION__, 0);
+        rd_kafka_t *consumer1, *consumer2, *consumer_1_to_destroy = NULL;
+        int32_t prev_generation_id1, next_generation_id1, prev_generation_id2,
+            next_generation_id2;
+        rd_kafka_topic_partition_list_t *prev_assignment1, *prev_assignment2,
+            *next_assignment1, *next_assignment2;
+
+        SUB_TEST_QUICK("%s", variation == 1 ? "with unsubscribe"
+                                            : "with new instance");
+
+        mcluster = test_mock_cluster_new(3, &bootstraps);
+        rd_kafka_mock_topic_create(mcluster, topic, 2, 3);
+
+        TEST_SAY("Creating consumers\n");
+        consumer1 = create_consumer(bootstraps, topic, "c1");
+        consumer2 = create_consumer(bootstraps, topic, "c2");
+
+        TEST_SAY("Subscribing consumers to topic \"%s\"\n", topic);
+        test_consumer_subscribe(consumer1, topic);
+        test_consumer_subscribe(consumer2, topic);
+
+        TEST_SAY("Waiting one EOF of consumer 1\n");
+        test_consumer_poll_exact("first consumer", consumer1, 0, 1, 0, 0,
+                                 rd_true, NULL);
+        TEST_SAY("Waiting one EOF of consumer 2\n");
+        test_consumer_poll_exact("second consumer", consumer2, 0, 1, 0, 0,
+                                 rd_true, NULL);
+
+        prev_generation_id1 = consumer_generation_id(consumer1);
+        prev_generation_id2 = consumer_generation_id(consumer2);
+        TEST_CALL_ERR__(rd_kafka_assignment(consumer1, &prev_assignment1));
+        TEST_CALL_ERR__(rd_kafka_assignment(consumer2, &prev_assignment2));
+        TEST_ASSERT(prev_assignment1 != NULL,
+                    "Expected assignment for consumer 1 before the change");
+        TEST_ASSERT(prev_assignment2 != NULL,
+                    "Expected assignment for consumer 2 before the change");
+
+        TEST_SAY("Unsubscribing consumer 1\n");
+        rd_kafka_mock_start_request_tracking(mcluster);
+        TEST_CALL_ERR__(rd_kafka_unsubscribe(consumer1));
+        test_mock_wait_matching_requests(mcluster, 1, 1000, is_api_key,
+                                         &api_key);
+        rd_kafka_mock_stop_request_tracking(mcluster);
+
+        if (variation == 2) {
+                /* Don't destroy it immediately because the
+                 * topic partition lists still hold a reference. */
+                consumer_1_to_destroy = consumer1;
+
+                TEST_SAY("Re-creating consumer 1\n");
+                /* Re-create the consumer with same group and instance id. */
+                consumer1 = create_consumer(bootstraps, topic, "c1");
+        }
+
+        TEST_SAY("Subscribing consumer 1 again\n");
+        test_consumer_subscribe(consumer1, topic);
+        test_consumer_wait_assignment(consumer1, rd_false);
+
+        next_generation_id1 = consumer_generation_id(consumer1);
+        next_generation_id2 = consumer_generation_id(consumer2);
+
+        TEST_ASSERT(next_generation_id1 == prev_generation_id1,
+                    "Expected same generation id for consumer 1, "
+                    "got %d != %d",
+                    prev_generation_id1, next_generation_id1);
+        TEST_ASSERT(next_generation_id2 == prev_generation_id2,
+                    "Expected same generation id for consumer 2, "
+                    "got %d != %d",
+                    prev_generation_id2, next_generation_id2);
+
+        TEST_CALL_ERR__(rd_kafka_assignment(consumer1, &next_assignment1));
+        TEST_CALL_ERR__(rd_kafka_assignment(consumer2, &next_assignment2));
+        TEST_ASSERT(next_assignment1 != NULL,
+                    "Expected assignment for consumer 1 after the change");
+        TEST_ASSERT(next_assignment2 != NULL,
+                    "Expected assignment for consumer 2 after the change");
+        TEST_ASSERT(!test_partition_list_and_offsets_cmp(prev_assignment1,
+                                                         next_assignment1),
+                    "Expected same assignment for consumer 1 after the change");
+        TEST_ASSERT(!test_partition_list_and_offsets_cmp(prev_assignment2,
+                                                         next_assignment2),
+                    "Expected same assignment for consumer 2 after the change");
+
+        rd_kafka_topic_partition_list_destroy(prev_assignment1);
+        rd_kafka_topic_partition_list_destroy(prev_assignment2);
+        rd_kafka_topic_partition_list_destroy(next_assignment1);
+        rd_kafka_topic_partition_list_destroy(next_assignment2);
+
+        RD_IF_FREE(consumer_1_to_destroy, rd_kafka_destroy);
+        rd_kafka_destroy(consumer1);
+        rd_kafka_destroy(consumer2);
+        test_mock_cluster_destroy(mcluster);
+
+        SUB_TEST_PASS();
+}
 
 int main_0102_static_group_rebalance(int argc, char **argv) {
+        /* TODO: check again when regexes
+         * will be supported by KIP-848 */
+        if (test_consumer_group_protocol_classic()) {
+                do_test_static_group_rebalance();
+        }
 
-        do_test_static_group_rebalance();
-
-        do_test_fenced_member();
+        if (test_consumer_group_protocol_classic()) {
+                do_test_fenced_member_classic();
+        } else {
+                do_test_fenced_member_consumer();
+        }
 
         return 0;
 }
+
+int main_0102_static_group_rebalance_mock(int argc, char **argv) {
+        TEST_SKIP_MOCK_CLUSTER(0);
+
+        if (test_consumer_group_protocol_classic()) {
+                TEST_SKIP(
+                    "Static membership isn't implemented "
+                    "in mock cluster for classic protocol\n");
+                return 0;
+        }
+
+        do_test_static_membership_mock(1);
+        do_test_static_membership_mock(2);
+        return 0;
+}
