@@ -61,7 +61,7 @@ static void do_test_fetch_unauth() {
 
   /* Create topic */
   const int partition_cnt = 3;
-  Test::create_topic(NULL, topic.c_str(), partition_cnt, 1);
+  Test::create_topic_wait_exists(NULL, topic.c_str(), partition_cnt, 1, 5000);
 
   /* Produce messages */
   test_produce_msgs_easy(topic.c_str(), 0, RdKafka::Topic::PARTITION_UA, 1000);
@@ -85,6 +85,9 @@ static void do_test_fetch_unauth() {
       "--topic '%s'",
       bootstraps.c_str(), topic.c_str());
 
+  /* Allow ACLs to propagate */
+  rd_sleep(1);
+
   Test::subscribe(c, topic);
 
   int auth_err_cnt = 0;
@@ -128,6 +131,7 @@ static void do_test_fetch_unauth() {
               "Expected exactly %d auth errors, saw %d", partition_cnt,
               auth_err_cnt);
 
+  c->close();
   delete c;
 
   Test::Say(tostr() << _C_GRN << "[ Test unauthorized Fetch PASS ]\n");
