@@ -65,7 +65,7 @@ do_test_produce_consumer_with_OIDC(const rd_kafka_conf_t *base_conf) {
         p1 = test_create_handle(RD_KAFKA_PRODUCER, rd_kafka_conf_dup(conf));
 
         topic = test_mk_topic_name("0126-oauthbearer_oidc", 1);
-        test_create_topic(p1, topic, 1, 3);
+        test_create_topic_wait_exists(p1, topic, 1, 3, 5000);
         TEST_SAY("Topic: %s is created\n", topic);
 
         test_produce_msgs2(p1, topic, testid, 0, 0, 1, NULL, 0);
