@@ -126,7 +126,7 @@ static void do_test_consume_batch_with_seek(void) {
         /* Produce messages */
         topic = test_mk_topic_name("0137-barrier_batch_consume", 1);
 
-        test_create_topic(NULL, topic, partition_cnt, 1);
+        test_create_topic_wait_exists(NULL, topic, partition_cnt, 1, 5000);
 
         for (p = 0; p < partition_cnt; p++)
                 test_produce_msgs_easy(topic, testid, p,
@@ -216,7 +216,7 @@ static void do_test_consume_batch_with_pause_and_resume_different_batch(void) {
         /* Produce messages */
         topic = test_mk_topic_name("0137-barrier_batch_consume", 1);
 
-        test_create_topic(NULL, topic, partition_cnt, 1);
+        test_create_topic_wait_exists(NULL, topic, partition_cnt, 1, 5000);
 
         for (p = 0; p < partition_cnt; p++)
                 test_produce_msgs_easy(topic, testid, p,
@@ -321,7 +321,7 @@ static void do_test_consume_batch_with_pause_and_resume_same_batch(void) {
         /* Produce messages */
         topic = test_mk_topic_name("0137-barrier_batch_consume", 1);
 
-        test_create_topic(NULL, topic, partition_cnt, 1);
+        test_create_topic_wait_exists(NULL, topic, partition_cnt, 1, 5000);
 
         for (p = 0; p < partition_cnt; p++)
                 test_produce_msgs_easy(topic, testid, p,
@@ -417,7 +417,7 @@ static void do_test_consume_batch_store_offset(void) {
         /* Produce messages */
         topic = test_mk_topic_name("0137-barrier_batch_consume", 1);
 
-        test_create_topic(NULL, topic, partition_cnt, 1);
+        test_create_topic_wait_exists(NULL, topic, partition_cnt, 1, 5000);
 
         for (p = 0; p < partition_cnt; p++)
                 test_produce_msgs_easy(topic, testid, p,
@@ -498,7 +498,7 @@ static void do_test_consume_batch_control_msgs(void) {
 
         producer = test_create_handle(RD_KAFKA_PRODUCER, conf);
 
-        test_create_topic(producer, topic, partition_cnt, 1);
+        test_create_topic_wait_exists(producer, topic, partition_cnt, 1, 5000);
 
         TEST_CALL_ERROR__(rd_kafka_init_transactions(producer, 30 * 1000));
 
