@@ -240,6 +240,11 @@ static RD_INLINE RD_UNUSED void rtrim(char *str) {
                 TEST_UNLOCK();                                                 \
         } while (0)
 
+#define TEST_SKIP_MOCK_CLUSTER(RET)                                            \
+        if (test_needs_auth()) {                                               \
+                TEST_SKIP("Mock cluster does not support SSL/SASL\n");         \
+                return RET;                                                    \
+        }
 
 void test_conf_init(rd_kafka_conf_t **conf,
                     rd_kafka_topic_conf_t **topic_conf,
@@ -695,6 +700,7 @@ void test_any_conf_set(rd_kafka_conf_t *conf,
                        const char *name,
                        const char *val);
 
+rd_kafka_topic_partition_list_t *test_topic_partitions(int cnt, ...);
 void test_print_partition_list(
     const rd_kafka_topic_partition_list_t *partitions);
 int test_partition_list_cmp(rd_kafka_topic_partition_list_t *al,
@@ -712,6 +718,11 @@ void test_create_topic(rd_kafka_t *use_rk,
                        const char *topicname,
                        int partition_cnt,
                        int replication_factor);
+void test_create_topic_wait_exists(rd_kafka_t *use_rk,
+                                   const char *topicname,
+                                   int partition_cnt,
+                                   int replication_factor,
+                                   int timeout);
 rd_kafka_resp_err_t test_auto_create_topic_rkt(rd_kafka_t *rk,
                                                rd_kafka_topic_t *rkt,
                                                int timeout_ms);
@@ -845,14 +856,28 @@ rd_kafka_resp_err_t test_delete_all_test_topics(int timeout_ms);
 void test_mock_cluster_destroy(rd_kafka_mock_cluster_t *mcluster);
 rd_kafka_mock_cluster_t *test_mock_cluster_new(int broker_cnt,
                                                const char **bootstraps);
+size_t test_mock_wait_matching_requests(
+    rd_kafka_mock_cluster_t *mcluster,
+    size_t num,
+    int confidence_interval_ms,
+    rd_bool_t (*match)(rd_kafka_mock_request_t *request, void *opaque),
+    void *opaque);
 
-
+void test_mock_cluster_member_assignment(rd_kafka_mock_cluster_t *mcluster,
+                                         int member_cnt,
+                                         ...);
 
 int test_error_is_not_fatal_cb(rd_kafka_t *rk,
                                rd_kafka_resp_err_t err,
                                const char *reason);
 
 
+const char *test_consumer_group_protocol();
+
+int test_consumer_group_protocol_classic();
+
+int test_consumer_group_protocol_consumer();
+
 /**
  * @brief Calls rdkafka function (with arguments)
  *        and checks its return value (must be rd_kafka_resp_err_t) for
