@@ -131,6 +131,21 @@ static RD_UNUSED void create_topic(RdKafka::Handle *use_handle,
   test_create_topic(use_rk, topicname, partition_cnt, replication_factor);
 }
 
+/**
+ * @brief Create a topic and wait for it to be available in metadata.
+ */
+static RD_UNUSED void create_topic_wait_exists(RdKafka::Handle *use_handle,
+                                               const char *topicname,
+                                               int partition_cnt,
+                                               int replication_factor,
+                                               int timeout) {
+  rd_kafka_t *use_rk = NULL;
+  if (use_handle != NULL)
+    use_rk = use_handle->c_ptr();
+  test_create_topic_wait_exists(use_rk, topicname, partition_cnt,
+                                replication_factor, timeout);
+}
+
 /**
  * @brief Delete a topic
  */
